import { DEFAULT_DOMAIN_CONFIG, DEFAULT_EXTENSION_DATA } from '~utils/constants';
import { formatDomainFromURL, validateSupport } from '~utils/domain';
import { storage } from '~utils/storage';
import type { DomainConfig, ExtensionData } from '~utils/types';

export default async function onBeforeRequest(details: chrome.webRequest.WebRequestBodyDetails) {
  const { tabId, type, url } = details;
  const location = new URL(url);
  const domain = formatDomainFromURL(location);

  if (tabId > -1 && type === 'main_frame') {
    const data = (await storage.get<ExtensionData>('data')) || DEFAULT_EXTENSION_DATA;

    if (!validateSupport(location.hostname, data.exclusions.domains)) {
      return;
    }

    const config = (await storage.get<DomainConfig>(domain)) || DEFAULT_DOMAIN_CONFIG;

    if (data.rules.length) {
      const rulesWithTabId = data.rules.map((rule) => ({
        ...rule,
        condition: { ...rule.condition, tabIds: [tabId] },
      }));

      chrome.declarativeNetRequest.updateSessionRules({
        addRules: config.on ? rulesWithTabId : undefined,
        removeRuleIds: data.rules.map((rule) => rule.id),
      });
    }
  }
}
