import { useStorage } from '@plasmohq/storage/hook';
import { useCallback, useEffect, useMemo, useState } from 'react';

import { DEFAULT_DOMAIN_CONFIG } from '~utils/constants';
import { formatDomainFromURL, validateSupport } from '~utils/domain';
import { storage } from '~utils/storage';
import type { DomainConfig, ExtensionData } from '~utils/types';

import { useTab } from './useTab';

export function useDomain(): UseDomainResult {
  const [domain, setDomain] = useState<string>('');
  const [config, setConfig] = useStorage<DomainConfig>({ instance: storage, key: domain });
  const [data] = useStorage<ExtensionData>({ instance: storage, key: 'data' });
  const tab = useTab();
  const tabId = tab ? (tab.id ?? -1) : -1;

  const isSupported = useMemo(
    () =>
      !!data?.exclusions.domains.length &&
      !!tab?.url &&
      validateSupport(new URL(tab.url).hostname, data.exclusions.domains),
    [data, tab]
  );

  const toggleStatus = useCallback(async () => {
    if (tabId > -1) {
      await setConfig((prev = DEFAULT_DOMAIN_CONFIG) => ({ ...prev, on: !prev.on }));
      await chrome.tabs.reload(tabId, { bypassCache: true });
    }
  }, [domain, tabId]);

  useEffect(() => {
    if (tab?.url) {
      setDomain(formatDomainFromURL(new URL(tab.url)));
    }
  }, [tab]);

  return {
    config: config || DEFAULT_DOMAIN_CONFIG,
    domain,
    isSupported,
    toggleStatus,
  };
}

export interface UseDomainResult {
  readonly config: DomainConfig;
  readonly domain: string;
  readonly isSupported: boolean;
  readonly toggleStatus: () => Promise<void>;
}
