import { useCallback, useEffect, useState } from 'react';

import { storage } from '~utils/storage';

async function getExclusions(): Promise<readonly Exclusion[]> {
  const result = await storage.rawGetAll();

  return Object.keys(result).flatMap((key) => (result[key]?.on === false ? [{ name: key }] : []));
}

export function useExclusions(): readonly Exclusion[] {
  const [exclusions, setExclusions] = useState<readonly Exclusion[]>([]);

  const handleStorageChange = useCallback(
    (
      changes: { [key: string]: chrome.storage.StorageChange },
      areaName: chrome.storage.AreaName
    ) => {
      if (areaName === 'local') {
        for (const key in changes) {
          if (key === 'data') {
            continue;
          }

          setExclusions((prev) => {
            if (changes[key].newValue?.on === false) {
              return [...prev, { name: key }];
            }

            return prev.filter((exclusion) => exclusion.name !== key);
          });
        }
      }
    },
    []
  );

  useEffect(() => {
    chrome.storage.onChanged.addListener(handleStorageChange);
    getExclusions().then(setExclusions);

    return () => {
      chrome.storage.onChanged.removeListener(handleStorageChange);
    };
  }, [handleStorageChange]);

  return exclusions;
}

export interface Exclusion {
  readonly name: string;
}
